<?php

namespace MSP\CashOnDelivery\Block\Adminhtml\Order\Creditmemo\Create;

use Magento\Framework\Pricing\PriceCurrencyInterface;

class Adjustments extends \Magento\Backend\Block\Template
{
    /**
     * Source object
     *
     * @var \Magento\Framework\DataObject
     */
    protected $_source;

    /**
     * Tax config
     *
     * @var \Magento\Tax\Model\Config
     */
    protected $_taxConfig;

    /**
     * @var PriceCurrencyInterface
     */
    protected $priceCurrency;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Tax\Model\Config $taxConfig
     * @param PriceCurrencyInterface $priceCurrency
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Tax\Model\Config $taxConfig,
        PriceCurrencyInterface $priceCurrency,
        array $data = []
    ) {
        $this->_taxConfig = $taxConfig;
        $this->priceCurrency = $priceCurrency;
        parent::__construct($context, $data);
    }

    /**
     * Initialize creditmemo agjustment totals
     *
     * @return $this
     */
    public function initTotals()
    {
        $parent = $this->getParentBlock();
        $this->_source = $parent->getSource();
        $total = new \Magento\Framework\DataObject(['code' => 'agjustments', 'block_name' => $this->getNameInLayout()]);
        $parent->removeTotal('shipping');
        $parent->removeTotal('adjustment_positive');
        $parent->removeTotal('adjustment_negative');
        $parent->addTotal($total);
        return $this;
    }

    /**
     * Get source object
     *
     * @return \Magento\Framework\DataObject
     */
    public function getSource()
    {
        return $this->_source;
    }

    /**
     * Get credit memo shipping amount depend on configuration settings
     *
     * @return float
     */
    public function getShippingAmount()
    {
        $source = $this->getSource();
        if ($this->_taxConfig->displaySalesShippingInclTax($source->getOrder()->getStoreId())) {
            $shipping = $source->getBaseShippingInclTax();
        } else {
            $shipping = $source->getBaseShippingAmount();
        }
        return $this->priceCurrency->round($shipping) * 1;
    }

    /**
     * Get label for shipping total based on configuration settings
     *
     * @return string
     */
    public function getShippingLabel()
    {
        $source = $this->getSource();
        if ($this->_taxConfig->displaySalesShippingInclTax($source->getOrder()->getStoreId())) {
            $label = __('Refund Shipping (Incl. Tax)');
        } elseif ($this->_taxConfig->displaySalesShippingBoth($source->getOrder()->getStoreId())) {
            $label = __('Refund Shipping (Excl. Tax)');
        } else {
            $label = __('Refund Shipping');
        }
        return $label;
    }
    
    
    
    public function getBaseMspCodAmount() {
        $source = $this->getSource();
        //we don't differentiate for including or excluding tax, because the amount is always the same
        //we handle this difference in total collector
        $mspCod = $source->getBaseMspCodAmount();
        return $this->priceCurrency->round($mspCod) * 1;
    }
    
    public function getMspCodLabel() {
        $source = $this->getSource();
        if ($this->_taxConfig->displayCartPricesInclTax($source->getOrder()->getStoreId())) {
            $label = __('Cash on Delivery fee (Incl. Tax)');
        } elseif ($this->_taxConfig->displayCartPricesBoth($source->getOrder()->getStoreId())) {
            $label = __('Cash on Delivery fee (Excl. Tax)');
        } else {
            $label = __('Cash on Delivery fee');
        }
        return $label;
    }
    
    public function isCod() {
        return ($this->getSource()->getOrder()->getPayment()->getMethod() == 'msp_cashondelivery');
    }

    /**
     * Get gift wrap value
     * @return mixed
     */
    public function getGWPrice() {
        return $this->getSource()->getOrder()->getGwBasePriceInclTax();
    }

    /**
     * Check if order has gift wrap
     * @return bool|int
     */
    public function orderHasGiftWrap() {
        $giftWrapId = $this->getSource()->getOrder()->getGwId();
        $giftWrapValue = ($this->getSource()->getOrder()->getGwPrice() + $this->getSource()->getOrder()->getGwTaxAmount());
        
        $res = !is_null($giftWrapId) && $giftWrapId > 0 && !is_null($giftWrapValue) && $giftWrapValue > 0;

        if(!$res) {
            if($giftWrapValue === 0) {
                $res = intval(0);
            }
            else {
                $res = false;
            }
        }

        return $res;
    }
}
