<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model\Total\Creditmemo;

use Magento\Sales\Model\Order\Creditmemo;
use Magento\Tax\Model\Config;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class CashondeliveryTax extends AbstractTotal
{
    
    /**
     * @var PriceCurrencyInterface
     */
    protected $priceCurrency;

    /**
     * Tax config
     *
     * @var \Magento\Tax\Model\Config
     */
    private $taxConfig;
    
    public function __construct(
        Config $taxConfig,
        PriceCurrencyInterface $priceCurrency,
        array $data = []
    ) {
        parent::__construct($data);
        $this->taxConfig = $taxConfig;
        $this->priceCurrency = $priceCurrency;
    }
    
    
    public function collect(Creditmemo $creditmemo)
    {
        $order = $creditmemo->getOrder();

        // Taxes calculation for gift wrap
        $gwTaxAmount = 0;
        $gwBaseTaxAmount = 0;
        $gwTaxInTotal = false;
        if($creditmemo->hasGwBasePriceInclTax() && ($creditmemo->getGwBasePriceInclTax() == 0 || $creditmemo->getGwBasePriceInclTax() < $order->getGwBasePriceInclTax()))
        {
            if($creditmemo->getGwBasePriceInclTax() < $order->getGwBasePriceInclTax())
            {
                $decreaseFactorGw = $creditmemo->getGwBasePriceInclTax() / $order->getGwBasePriceInclTax();
                $gwTaxAmount = $this->priceCurrency->round($order->getGwTaxAmount() * $decreaseFactorGw);
                $gwBaseTaxAmount = $this->priceCurrency->round($order->getGwBaseTaxAmount() * $decreaseFactorGw, 'base');
                $gwTaxInTotal = true;
            }

            $creditmemo->setGwTaxAmount($gwTaxAmount);
            $creditmemo->setGwBaseTaxAmount($gwBaseTaxAmount);

            if($creditmemo->getGwBasePriceInclTax() == 0)
            {
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $order->getGwTaxAmount());
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $order->getGwBaseTaxAmount());
            }
            else
            {
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $gwTaxAmount);
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $gwBaseTaxAmount);
            }
        }

        if (!$order->getBaseMspCodAmount() || $order->getBaseMspCodAmount() <= 0) {
            $creditmemo->setMspCodTaxAmount(0);
            $creditmemo->setBasePaymentTaxAmount(0);
            return $this;
        }
        
        $decreaseFactor = $creditmemo->getBaseMspCodAmount() / $order->getBaseMspCodAmount();
        $mspCodTaxAmount = $this->priceCurrency->round($order->getMspCodTaxAmount() * $decreaseFactor);
        $baseMspCodTaxAmount = $this->priceCurrency->round($order->getBaseMspCodTaxAmount() * $decreaseFactor, 'base');
        $creditmemo->setMspCodTaxAmount($mspCodTaxAmount);
        $creditmemo->setBasePaymentTaxAmount($baseMspCodTaxAmount);
        
        /*
         * Fix taxes calculation for MspCod
         * MspCod tax calculation was fixed for entire value of MspCod or if is setted to 0.
         * To check if MspCod value is included in this range.
         */
        if($creditmemo->getBaseMspCodAmount() == 0)
        {
            $mspCodTaxAmount = $order->getMspCodTaxAmount();
            $baseMspCodTaxAmount = $order->getBaseMspCodTaxAmount();
            $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $order->getMspCodTaxAmount());
            $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $order->getBaseMspCodTaxAmount());
        }
        else
        {
            if($mspCodTaxAmount < $order->getMspCodTaxAmount())
            {
                $creditmemo->setTaxAmount($creditmemo->getTaxAmount() - $mspCodTaxAmount);
            }
            if($baseMspCodTaxAmount < $order->getBaseMspCodTaxAmount())
            {
                $creditmemo->setBaseTaxAmount($creditmemo->getBaseTaxAmount() - $baseMspCodTaxAmount);
            }
        }


        if (!$this->taxConfig->displayCartPricesExclTax($order->getStoreId())) {
            $creditmemo->setGrandTotal($creditmemo->getGrandTotal() - $mspCodTaxAmount);
            $creditmemo->setBaseGrandTotal($creditmemo->getBaseGrandTotal() - $baseMspCodTaxAmount);
            
        }

        return $this;
    }
}
